function handles=putonsurf(X,Y,Z,base1,base2,eyeset,cellarray,varargin)
%PUTONSURF(X,Y,Z,base1,base2,eyecell,cellarray,OPTIONS)
%
% Adds graphic elements of three possible types to parametrized surfaces. Examples:
%
% putonsurf(X,Y,Z,base1,base2,{az,el,[]},{{'cross',x,y}})
% putonsurf(X,Y,Z,base1,base2,{az,el,[]},{{'pointline',x,y,'col',[1 1 0]}})
% putonsurf(X,Y,Z,base1,base2,{az,el,[]},{{'surfline',x,y}},'mode',2)
%
% X,Y and Z are the position matrices for the background surface. 
%
% base1 and base2 are vectors describing the parameter values of the
% surface nodes. base1 must have the same number of elements as there
% are rows in X and Y
%
% eyeset={az,el,radius} determines the position of the eye
% so that the new elements may be lifted slightly towards the eye. 
%
% cellarray={  {type,x,y,OPTIONS1},...
%              {type,x,y,OPTIONS2},...
%              {type,x,y,OPTIONS3}.... and so on}
% is an array of cells of any number of rows. 
% Each cell (row) corresponds to one new graphical element.
%
%            type can be 'surfline', 'pointline' or 'cross'
%
%            x and y are vectors describing the parametervalues
%            for the line or cross. In the case of the cross the 
%            vectors have length 1.
%
%            OPTIONS_NR are of the type 'parameter', value, 'parameter',
%            value ... What parameters that can be set depends on the
%            type of object. See rjxsurfcross.m, rjxplot3.m and rjxsurfline
%            respectively
%
%
% Options are of the type 'parameter', value, 'parameter', value ...
% default={
%    'scale',    1;
%    'offset',   [0 0 0];
%    'mode',     1}
%
% This program now does three quite different things. It is however
% simple to split it into three separate programs if so desired.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

default={
    'scale',    1;
    'offset',   [0 0 0];
    'mode',     1;
    'edgecut','off';    %Cutting objects if they pass the basesurface
    'fixnormal',[];     %If plane -  one may specify the normal in
                        %advance
    'normalepsilon',0;  %Lifts object in the direction of the normal
    'autoendplane','off'        %Set 'on' to automatically calculate
                                %endplanenormals  (for surfline only)
	};      

eval(setvalues)

%%%%%%% Altering element specific default values %%%%%%%%%%

%pointline
rjxplot3_defaultalter={         
	    'col',    [0 1 1],...
	    'epsilon',   0.01,...
		   };

%cross
rjxsurfcross_defaultalter={      
    'col',     [0 1 1],...
    'width', 0.01,...
    'thick', 0.1,...
		   };

%surfline	
rjxsurfline_defaultalter={       
    'col',     [0 1 1],...
    'arrow',    'off',...     
    'startwidth', 0.1,...
    'endwidth', 0.1,...
		   };

%%%%%%%%%%  The stuff to send along %%%%%%%%%%%%%%%%%%%%%%%%%

rjset={...
    'scale',  scale,...
    'offset', offset ,... 
    'mode',   mode,...
    'az',     eyeset{1},...
    'el',     eyeset{2},...
    'radius', eyeset{3}
};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if length(eyeset)~=3
    error('RJ: Send eyset of the form {az,el,radius}' )
end

if size(cellarray,1)~=1 & size(cellarray,2)~=1
    error('RJ: cellarray must be a single column or a single row')
end

if size(X,1)~=length(base1)
    error('RJ: No of elements in base1 not corresponding to no of rows in X')
end

if size(X,2)~=length(base2)
    error('RJ: No of elements in base2 not corresponding to no of columns in X')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hold on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% The real stuff starts here %%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

handles=[];
for nr=1:length(cellarray)
    
    in=cellarray{nr};
    if length(in)<3
        error('RJ: Cellarray element form {type,x,y,OPTIONS}' )
    end
    
    s=length(in)-3;
    if floor(s/2)~=s/2
        error('RJ: Options in cellarray element, even no of elements')
    end
    
    if s>1
        newvarargin=in(4:length(in));
    else
        newvarargin=[];
    end
    
    type=in{1};
    x=in{2};
    y=in{3};
    
    if length(x)~=length(y)
	error('RJ: type,x,y; x and y different lengths')
    end
    
    rjempty=0;
    if isequal(edgecut,'on')
        [x y]=cutthecrap(base1,base2,x,y);
        if isempty(x) | isempty (y)
            rjempty=1;
        end
    end
    
    %%%%%%%%%%% Finding positions and normals %%%%%%%%%
    
    position=zeros(length(x),3);           %Initializing
    normal=zeros(length(x),3);             %Initializing
    
    ivec=zeros(size(x));
    jvec=zeros(size(x));
    
    for nr=1:length(x);
        i=decimalindex(x(nr),base1);
        j=decimalindex(y(nr),base2);
        ivec(nr)=i;
	jvec(nr)=j;
	position(nr,:)=rjfinddecimalposition(i,j,X,Y,Z);
        if ~isequal(in{1},'pointline')
            if isempty(fixnormal)
                normal(nr,:)=rjfinddecimalnormal(i,j,X,Y,Z);
            else
                normal(nr,:)=fixnormal;
            end	    
        end
    end
    
    
    if ~isequal(normalepsilon,0)
	for nr=1:length(x);
	    position(nr,:)=position(nr,:)+normal(nr,:)*normalepsilon;
	end
    end
    
    
    
    %%%%%%%%%%%%%% Fixing automatic endplanenormals %%%%%%%%%%%%%%%%%
    
    %I am currently assuming that we are not near a corner
    
    if isequal(autoendplane,'on')
	
	istart=floor(ivec(1));
	jstart=floor(jvec(1));
	iend=floor(ivec(length(ivec)));
	jend=floor(jvec(length(jvec)));
	isevec=[istart,iend];
	jsevec=[jstart,jend];
	
	maxi=size(X,1);
	maxj=size(X,2);
	
	
	for anr=1:2
	    
	    ii=isevec(anr);
	    jj=jsevec(anr);
	    
	    if ii==1
		a1=0;b1=1;a2=1;b2=0;
	    end
	    
	    if ii==maxi
		a1=0;b1=1;a2=-1;b2=0;
	    end
	    
	    if jj==1
		a1=1;b1=0;a2=0;b2=1;
	    end
	    
	    if jj==maxj
		a1=1;b1=0;a2=0;b2=-1;
	    end
	
	    
	    pos0=[X(ii,jj),Y(ii,jj),Z(ii,jj)];
	    pos1=[X(ii+a1,jj+b1),Y(ii+a1,jj+b1),Z(ii+a1,jj+b1)];
	    pos2=[X(ii+a2,jj+b2),Y(ii+a2,jj+b2),Z(ii+a2,jj+b2)];
	    delta1=pos1-pos0;
	    delta2=pos2-pos0;
	    temp=cross(delta1,delta2);
	    temp=cross(delta1,temp);
	    planenormalset{anr}=rjnormalize(temp);
	end
	startplanenormal=planenormalset{1};
	endplanenormal=planenormalset{2};
    else
	startplanenormal=[];
	endplanenormal=[];
    end
    
   
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %%%%%%%%%%%%% The plots %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    
    if isempty(newvarargin)
	newvarargin={};
    end
    
    %%%%%%%%%%%%% Pointline %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           
    if isequal(type,'pointline') & rjempty==0
	h1=rjxplot3(position(:,1),position(:,2),position(:,3),'k',...
		    rjxplot3_defaultalter{:},rjset{:},newvarargin{:});
	handles=[handles h1];
	
    %%%%%%%%%%%%%% Surface line %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
    elseif isequal(type,'surfline') & rjempty==0
	h2=rjxsurfline(position,normal,rjxsurfline_defaultalter{:},rjset{:},newvarargin{:},...
		       'startplanenormal',startplanenormal,'endplanenormal',endplanenormal);
	handles=[handles h2];
	
    %%%%%%%%%%%%% Cross %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
    elseif isequal(type,'cross') & rjempty==0  	
	h3=rjxsurfcross(position,normal,[0 0 1], ...
			rjxsurfcross_defaultalter{:},rjset{:},newvarargin{:}); ...
	   %[x y]=cutthecrap(base1,base2,x,y);   %Took this away
           %... seems to destroy evrything and seems meaningless.
	handles=[handles h3]; 
    
    %%%%%%%%%%%%% Else nothing  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    elseif rjempty==0 
	error(['RJ: Unknown type: ' type])
    end  
    
    s=size(colormap);    
    caxis([1 s(1)]);	
end











