function out=rjxminkowski(X,Y,Z,xbase,ybase,eyeset,varargin)
%MINKOWSKI(X,Y,Z,xbase,ybase,eyeset,OPTIONS)
% 
% Puts a Minkowski system, including coordinate axes and lightcone, on a
% curved surface. Meant to be used in the absolute scheme of spacetime visualization.
%
% X,Y,Z are the matrixes describing the surface where to put the
% minkowski system. xbase and ybase are the coordinate base vectors for
% the surface. eyeset={az,el,radius} where radius=[] corresponds to the
% eye at infinity - this enables lifting of the minkowski system towards
% the observers eye.
%
% There are several more or less refined ways to specify the appearance
% of the minkowski system, using the OPTIONS.
% 
% If nothing at all is specified in the options, a flat minkowskisystem is
% set in the coordinate center of the surface
%
% The most refined, and simple to use, method is when 'setlightcone' is
% set to 'auto'. If the generators of the timeaxis is directed along a
% coordinate axis it is then trivial to get the minkowski system there, e.g: 
%
% hvec=rjxminkowski(X,Y,Z,tbase,xbase,eyeset,...
%			      'tline',{t,x},'setlightcone','auto',...
%			      'msize',1.3,'autodir',2,...
%			      'maxes','on','epsilon',0.01,'mode',1);
%
% t and x are the coordinates for the origin of the minkowski
% system. 'autodir' can be 1,2,3,4 and stands for the four trivial
% coordinate directions. If the generating field is not any of the four
% trivial ones - one may specify it by adding
% 'generatorstring','functionname' where functionname is a function that
% takes two arguments (t,x) and returns a generator direction
% (dt,dx). Then the axes and the lightcone is calculatad to be exactly
% paralell, orthogonal or at 45 degrees respectively to the local
% generator. This is done via the function generateminkowski, which calls
% the function findline. If one desires one can lock the generator at the
% origin to extend over the whole lightcone by adding 'lockorigin','on'.
% This makes all the lines in the system coordinate straight.
% For a flat surface with a non trivial generating field this then gives
% exactly triangular lightcones, but that aren't nececarily exactly at 45
% degrees to the generating field (except at the origin). 
% 
% If one for some reason find it difficult to give a function for the
% generators (the function may depend on the shape of the surface e.g)
% one can produce a coordinate straight system in the direction specified
% by 'fixdir' of the form [dt,dx]. It overrules any specified generator
% function and locks the origin.
% 
% There are options to deal with closed boundary conditions, say that you
% want to put a lightcone at the surfacejoining. One can set 'yclosed'
% and 'xclosed' to 'on' to acchieve this but it is currently
% unstable. The reason lies in findline.m. When the line passes a border
% it is 'renormalized' immediately in coordinate space to fit within the borders
% Effectively this creates a long straight line over
% coordinate space. For the axes this is no problem - once put on the
% surface the result is perfect. For the lightcone it is quite different
% however since trianglegrid searches for intersection points between the
% edges of the triangle. Naturally the straight line going across the
% coordinate space may provide a false intersection point. What one may
% do is to let findline continue on with the coordinate line outside of
% the closed boundary (taking generators and metric from the
% corresponding internal point). Then this is sent to trianglegrid. But
% then one would have to put a closedboundary condition on the
% putonsurf.m file. Or one could do it immediately in trianglegrid,
% after the matrix has been created one may move the outside points back
% in. But really - I guess it should be done in the putonsurf so one may
% use it in other instances. It will have to wait for now though :)
%
% The setlightcone 'auto' is in virtually all espects better than the
% other, older options listed below. If you set some old options different
% than their default values you may destroy the 'auto' function though.
% Below is an old version of help, for the older options that are kept
% for backwards compability. 
%
% REGARDING THE MINKOWSKI AXES SYSTEM FOR STANDARD SURFACES:
%
% 'tline' contains the coordinate vectors of the main time arrow of the
% minkowski system. If this corresponds to a longer
% vector (line) on the surface than that specified by msize, the vector will be cut to
% the proper length. If it is shorter - the vector will be extended (as a
% straigh line in space) to the proper length. 
% 
% 'rightline' is analogous to 'tline', but refers to the rightgoing
% spatial coordinate line. If empty ({[],[]}) it is calculated
% to be proper orthogonal to the time axis - and to be
% coordinate straight.  
%
% 'leftline' is analogous to 'rightline', but refers to the leftgoing
% spatial coordinate line. If empty ({[],[]}) it is calculated
% to be a coordinate inversion of 'rightline'
%
% For the special case of a rotational surface where the minkowski
% directions are either purely azimuthal or directed along the surface
% one needs only to specify the direction (1,2,3,4) by 'dirmode', and the
% starting position (the first elements of 'tline') and the rest is automatic.
%
%
% REGARDING THE LIGHTCONE
%
% There are three different ways to specify the lightcone. 
%
% If 'setlightcone' is 'on', a flat lightcone tangent to the surface is
% plotted. 
%
% If setlightcone is 'curved' a lightcone that is following the
% surface snuggly is produced - but this only works on rotational
% surfaces when the lightcone is opening up directly in the azimuthal
% direction or along the surface. For this feature one must also specify
% 'rotset'= {r,z} - for the rotational surface. The direction (1,2,3,4)
% of the lightcone is specified by 'dirmode'
%
% The third opportunity lies in explicitly specifying the coordinate
% lines corresponding to the edges of the lightcone. This is specified
% with 'rightphoton', 'leftphoton' and 'topline'.

default={
    'tline',{[],[]};         %The time axis line                  {xvec,yvec}
    'rightline',{[],[]};     %The rightgoing spatial axis line    {xvec,yvec}
    'leftline',{[],[]};      %The leftgoing spatial axis line     {xvec,yvec}
    'setlightcone', 'on';    %'on', 'curved', 'off', 'coorextend', 'auto'
    'msize',0.2;             %The proper size of the system
    'arrowcol', [1 0 1]*0.75*0;     %Color of coordinate arrows
    'lightconecol', [1 1 0]; %Color of lightcone
    'transparency', 0.8;     %Transparency of the light cone
    'leftright','on';        %Allows extra spatial axis. Automized if none specified
    'mode',1;                %The ordinary mode for permutation of x,y and z axes.
    'epsilon',0.3;           %Lifting of minkowski system towards the eye
    'dirmode',[];            %[],1,2,3,4
    'rotset',[];             %If rotsurface rotset={rvec,zvec}
    'rightphoton',{};        %The rightgoing photon line          {xvec,yvec}
    'leftphoton',{};         %The leftgoing photon line           {xvec,yvec} 
    'topline',{};            %The top line connecting the photons {xvec,yvec}
    'number',10;             %The number of points in the axis, assuming dirmode~=[]
    'maxes','on'             %Turns on or off the minkowski axes
    'lapnumber',1;           %For rotational surfaces, the number of
                             %laps, for curved lightcone.
    'number',10;             %Number of points in lightcone, when
                             %'setlightcone' is auto
    'generatorstring','';    %A function that returns the
                             %generator(direction) given the point, when
                             %'setlightcone' is auto 
    'lightconefactor',0.75   %How high up the lightcone should extend
    'lockorigin','off';      %When on, and setlightcone auto - locks the
                             %metric and generators at the origin to
                             %apply everywhere in the system. Makes the
                             %system coordinate straight and proper
                             %orthogonal at the origin.
    'autodir',[];            %1,2,3,4 same as dirmode, but for lightcone
                             %'auto'
    'extraangle',0;          %Add extra proper angle to mink-system assuming
                             %ligthcone 'auto'
    'xcentershift','off';    %Shifts the lightcone (in coordinates), after it is
                             %created, to make the middle lightcone point,
                             %rather than the origin,  sit at the specified
                             %x-coordinate (mainly for intrinsically
                             %flat surfaces)
    'ycentershift','off';    %Same as xcentershift, but for y
    'xclosed','off';         %Closed boundary conditions
    'yclosed','off';         %Closed boundary conditions
    'fixdir',[];             %If non-zero it overrules the generator
			     %function for gendir and locks the origin
                             %(assuming lightcone 'auto').
    'extrafactor',2;         %How far extra to extend the lines, they
                             %will be cut later assuming 'auto'. (Passed
                             %on to generateminkowski)
    'trivialmode','off';     %NOT IMPLEMENTED YET - BIT TROUBLESOME. 
			     %Set 'on' to speed up the intersection process
                             %if one of the lightcone lines are constant
                             %or almost constant in one of the coordinates
	};

eval(setvalues);
hold on
if isequal(leftright,'on');spacefactor=lightconefactor;else;spacefactor=0.75;end

%%%%%%%%%%%% Sets up an on-surface system including the lightcone %%%%%%%%%%%%%%%%%%%

if isequal(setlightcone,'auto')
    if isempty(generatorstring)
	if isempty(autodir)
	    disp(['RJ: No generator function specified, no autodir specified, reverting to standard' ...
		  ' generator corresponding to autodir 1'])
	    autodir=1;
	end
	if autodir==1;generatorstring='standardgenerator1';end
	if autodir==2;generatorstring='standardgenerator2';end
	if autodir==3;generatorstring='standardgenerator3';end
	if autodir==4;generatorstring='standardgenerator4';end
    end
        
    temp1=tline{1};
    temp2=tline{2};
    x=temp1(1);
    y=temp2(1);
    
    [tline,rightline,leftline,rightphoton,leftphoton,topline]=...
	generateminkowski(X,Y,Z,xbase,ybase,x,y,generatorstring,'number', ...
			  number,'msize',msize,'lightconefactor',lightconefactor,...
			  'lockorigin',lockorigin,'extraangle',extraangle,...
			  'xcentershift',xcentershift,'ycentershift',ycentershift,...
			  'yclosed',yclosed,'xclosed',xclosed,'fixdir',fixdir,'extrafactor',extrafactor);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

xvec1=tline{1};
yvec1=tline{2};
xvec2=rightline{1};
yvec2=rightline{2};
xvec3=leftline{1};
yvec3=leftline{2};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isequal(maxes,'on')
    if ~isempty(xvec1) | ~isempty(yvec1) 
	if xvec1(1)>max(xbase) | xvec1(1)<min(xbase) | yvec1(1)>max(ybase) | yvec1(1)<min(ybase)  
	    error('RJ: minkowski out of bounds')
	end
    end
end

%%%% Just throwing out a system in the center if nothing specified %%%%%%%%%%%

if isempty(xvec1) | isempty(yvec1)
    %xvec1=(min(xbase)+max(xbase))/2;
    %yvec1=(min(ybase)+max(ybase))/2;
    temp=(min(xbase)+max(xbase))/2;
    xvec1=[temp,temp+0.00001];
    temp=(min(ybase)+max(ybase))/2;
    yvec1=[temp,temp];
    setlightcone='on';  %Not sure this is the most natural way but ...
end

%%%%%%% Fixing an automized orthogonal axis %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isempty(xvec2) | isempty(yvec2)
    dx=xvec1(length(xvec1))-xvec1(1);
    dy=yvec1(length(yvec1))-yvec1(1);
    [guu guv gvv]=getmetric(X,Y,Z,xbase,ybase,xvec1(1),yvec1(1));
    [dxort,dyort]=orthogonalvector(dx,dy,guu,guv,gvv);
    xvec2=linspace(xvec1(1),xvec1(1)+dxort,length(xvec1));
    yvec2=linspace(yvec1(1),yvec1(1)+dyort,length(yvec1));
end

%%%%%%% Setting up the system along the main axes (typically for rotational surfaces) %%%%

if  (length(xvec1)<2 | length(yvec1)<2) & isempty(dirmode) 
    dirmode=1;    %Dirmode ~=[] means rotational surface assumed
end

cutlengthfactor=1;
if ~isempty(dirmode) 
    rcell=rotminkowski(X,Y,Z,xbase ,ybase    ,xvec1(1) ,yvec1(1)  ,dirmode, msize,...
		       'spacefactor',spacefactor,'number',number);
         %rotminkowski(X,Y,Z,a_base,phi_base ,a_start  ,phi_start ,conemode,msize);
    xvec1=rcell{1};yvec1=rcell{2};
    xvec2=rcell{3};yvec2=rcell{4};
    xvec3=rcell{5};yvec3=rcell{6};
    cutlengthfactor=[];  %Otherwise it is extended a very very small distance
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cutlength=msize;
width=0.08*msize;
thcd={};


if ~isequal(maxes,'off')

    %%% The time axis %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    thcd{length(thcd)+1}={'surfline',xvec1,yvec1,'epsilon',epsilon,'cutlength',msize*cutlengthfactor,...
		    'arrow','curved','width',width,'col',arrowcol,'backextend',width/2};
    
    %%% The spatial axes %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    thcd{length(thcd)+1}={'surfline',xvec2,yvec2,'epsilon',epsilon,...
		    'cutlength',msize*spacefactor*cutlengthfactor,'width',width,...
		    'arrow','curved','col',arrowcol,'backextend',width/2};
    
    if isempty(xvec3) | isempty(yvec3)  %Then extend xvec2 backwards to form xvec3 
	xvec3=2*xvec2(1)-xvec2;
	yvec3=2*yvec2(1)-yvec2;
    end
    
    if isequal(leftright,'on')
	thcd{length(thcd)+1}={'surfline',xvec3,yvec3,'epsilon',epsilon,...
		    'cutlength',msize*spacefactor*cutlengthfactor,'width',width,'arrow','curved',...
		    'col',arrowcol,'backextend',width/2};
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% The light cone %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isequal(setlightcone,'on') & isempty(rightphoton) | isequal(setlightcone,'plain')
thcd{length(thcd)+1}={'surfline',xvec1,yvec1,'epsilon',epsilon*5/3,'cutlength',...
		    msize*lightconefactor,'arrow','off','startwidth',0, ...
		    'endwidth',2*msize*lightconefactor,'col', ...
		    lightconecol};
conenr=length(thcd);
end


%%%%%%%% Creates an on-surface lightcone that is coordinate straight %%%%%%%%%%%%%%%%
%%%%%%%% Not used - use setlightcone 'auto' and lockorigin 'on' instead %%%%%%%%%%%%%

if isequal(setlightcone,'coordextend')
    dx=xvec1(length(xvec1))-xvec1(1);
    dy=yvec1(length(yvec1))-yvec1(1);
    [guu guv gvv]=getmetric(X,Y,Z,xbase,ybase,xvec1(1),yvec1(1));
    [dxort,dyort]=orthogonalvector(dx,dy,guu,guv,gvv);
    rightphoton{1}=linspace(xvec1(1),xvec1(1)+dxort+dx,length(xvec1));
    rightphoton{2}=linspace(yvec1(1),yvec1(1)+dyort+dy,length(yvec1));
    leftphoton{1}=linspace(xvec1(1),xvec1(1)+dxort+dx,length(xvec1));
    leftphoton{2}=linspace(yvec1(1),yvec1(1)-dyort+dy,length(yvec1));
    distvec=distonsurf(X,Y,Z,xbase,ybase,xvec1,yvec1);
    xstart=findvalue(msize*lightconefactor,distvec,xvec1);
    ystart=findvalue(msize*lightconefactor,distvec,yvec1);
    topline{1}=linspace(xstart-dxort,xstart+dxort,length(xvec1));
    topline{2}=linspace(ystart-dyort,ystart+dyort,length(yvec1));
end

mo=min(ybase);                                 % Making sure that lightcone gets a phi basis
ms=2*pi/(max(ybase)-min(ybase))*lapnumber;

if ~isempty(rightphoton) & ~isequal(setlightcone,'plain')
    mo=0;
    ms=1;
    az=eyeset{1};el=eyeset{2};
    h1=lightcone(X,Y,Z,xbase,(ybase-mo)*ms,...
		 rightphoton{1},(rightphoton{2}-mo)*ms,...
		 leftphoton{1},(leftphoton{2}-mo)*ms,...
		 topline{1},    (topline{2}    -mo)*ms,'epsilon',epsilon*5/3,...
		 'col',lightconecol,'az',az,'el',el,'radius',eyeset{3},'mode',mode);
 
    set(h1,'FaceAlpha',transparency);
    no=1;
else
    no=0;
end



if isequal(setlightcone,'curved') & no~=1
    
    if isempty(rotset)
	error('RJ: You must specify rotset={rvec,zvec} in the OPTIONS');
    end
    
    if isempty(dirmode)
	warning('You might want to set the dirmode. Defaulting to 1')
	dirmode=1;
    end
    
    h1=rotlightconesurf(X,Y,Z,rotset{1},rotset{2},xbase,(ybase-mo)*ms,xvec1(1),(yvec1(1)-mo)*ms,...
			dirmode,msize,'epsilon',epsilon*5/3,'az',eyeset{1},'el',eyeset{2},...
			'radius',eyeset{3},'col',lightconecol,'mode',mode);
    set(h1,'FaceAlpha',transparency);
end

%%% Putting it all on the surf %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isempty(thcd)
    h=putonsurf(X,Y,Z,xbase,ybase,eyeset,thcd,'edgecut','on','mode', ...
		mode);
end

if ~exist('h')
	h=[];
end

if exist('conenr')
    set(h(conenr),'FaceAlpha',transparency)
end

if exist('h1')    %Making sure that the light cone is allways the last out
    out=[h h1];
else
    out=h;
end
















