function [out,out1,out2]=rjxsurf(A,B,C,D,varargin)
% RJXSURF(X,Y,Z,C,Options)
%
% The options with default values are:
% default={
%     'scale',    1;
%     'offset',   [0 0 0];
%     'mode',     1;
%     'az',       []
%     'el',       [];
%     'radius',   [];     
%     'epsilon',  0.01;
%     'setlines', 'off';
%     'linesep1', 2;
%     'linesep2', 2;
%     'linestyle','b';
%     'linestyle1','';
%     'linestyle2','';
%     'farg',     'absolute'
% };
%
% Works like SURF but with extra parametrical options.
% 'scale' , 'offset' and 'mode' sets the scale, displacement and
% axis orientation respectively.
% 
% This can be usefull for instance when combining several
% images from different programs to form a single picture.
% 
% 'az' and 'el' are the azimuthal and elevation looking angles.
% If these are both non-empty arguments (in degrees), the surface
% is shifted towards the eye by a distance epsilon. If the 'radius'
% argument is empty, the eye is assumed to be at infinity, otherwise
% at the given radius. The 'epsilon' has a default value but
% be set by the parameters.
% 
% This is usefull when one surface, such as a surface created by
% rjxsurfline, should closely follow some other surface (i.e 
% a trajectory on a spacetime hat) but be on top.
% 
% Notice that one can also send arguments, partially or completely,
% as a cell vector using the option 'settings'.
%
% An added feature is the possibility to draw, not every line of 
% the surface, but as specified by 'linesep1' and 'linesep2'.
% To use this one must set 'lines','on'. If one does this
% any liftings towards the eye from 'az','el','epsilon' and 'radius'
% refers to the lines and not the surface itself

if rem(nargin,2)==1,
   error('RJ: Property value pairs expected.')
end  

% The default values

default={
    'scale',    1;
    'offset',   [0 0 0];
    'mode',     1;
    'az',       []
    'el',       [];
    'radius',   [];     
    'epsilon',  0.01;
    'setlines', 'off';
    'linesep1', 2;
    'linesep2', 2;
    'linestyle','y';
    'linestyle1','';
    'linestyle2','';
    'farg',     'absolute';
    'toplot3',{};
    'col', [];
    'linecol', [];
    'eyeset',[];    %{az,el,radius}
    'distort',[];
    'distortset',{};
};

eval(setvalues)

if ~isempty(col)
    farg=col;
end

if ~isempty(eyeset)
    az=eyeset{1};
    el=eyeset{2};
    radius=eyeset{3};
end

if isequal(radius,inf)
    warning('RJ: Setting radius []')
    radius=[];
end

if ~isempty(distort)
    [A B C]=eval([distort '(A,B,C,distortset{:})']);
end

%%%%% Fixing stuff related to holding the plot %%%%

initialhold=ishold;
if ~initialhold;clf;end
hold on

%%%%% Prompting if one forgot az or el %%%%%%%%%%%%

if ~isequal(epsilon,0.01)
    if isempty(az) | isempty(el)
        %disp('RJ: You must send az and el to lift epsilon towards the eye')
    end
end

%%%%%%%%%%%%%%% The lines %%%%%%%%%%%%%%%%%%%%%%%%%

if isequal(setlines,'on')
    if isempty(linestyle1);linestyle1=linestyle;end
    if isempty(linestyle2);linestyle2=linestyle;end
    
    if isempty(az) | isempty(el)
        warning('RJ: No lifting of lines, to lift specify ''az'' and ''el'' ')
    end
    X=A; Y=B; Z=C;
    s=size(Z);
    to_lines={...
	'scale', scale,...
	'offset', offset ,...
	'mode', mode,... 
	'az', az,...  
	'el', el,...    
	'radius', radius ,...   
	'epsilon', epsilon,...
	'toplot3',toplot3};
    
    XL=X([1:linesep1:s(1)],:);
    YL=Y([1:linesep1:s(1)],:);
    ZL=Z([1:linesep1:s(1)],:);
    out1=rjxplot3(XL',YL',ZL',linestyle1,'settings',to_lines);

    XL=X(:,[1:linesep2:s(2)]);
    YL=Y(:,[1:linesep2:s(2)]);
    ZL=Z(:,[1:linesep2:s(2)]);
    
    out2=rjxplot3(XL,YL,ZL,linestyle2,'settings',to_lines);
    az=[];
    el=[];
    radius=[];
    if ~isempty(linecol)
	set([out1;out2],'Color',linecol)
    end
end

%%%%%%%%%%%%%% The surface %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[X Y Z]=rjxset(A,B,C,scale,mode,offset);

el=el/180*pi;
az=az/180*pi;

norm=[cos(el)*sin(az) -cos(el)*cos(az) sin(el)];  %az is zero at the negative y-axis

if ~isempty(radius)
    r0=radius*norm;
end

if isempty(radius)
    if isempty(az) | isempty(el)
        %Do nothing to X,Y and Z
    else
        v=epsilon*norm;
        X=X+v(1);
        Y=Y+v(2);
        Z=Z+v(3);
    end       
else
    if isempty(az) | isempty(el)
        error('RJ: You must specify the angles ''az'' and ''el'' when you give a ''radius''.')
    end
        DX=r0(1)-X;
        DY=r0(2)-Y;
        DZ=r0(3)-Z;
        DR=sqrt(DX.^2+DY.^2+DZ.^2);
        DXnorm=DX./DR;
        DYnorm=DY./DR;
        DZnorm=DZ./DR;
        X=X+DXnorm*epsilon;
        Y=Y+DYnorm*epsilon;
        Z=Z+DZnorm*epsilon;
end

%The latter alternative is there to enable lifting of the points 
%towards the eye when using perspective viewing - usable
%for many spacetime hat applications :)

if ~isequal(farg,'absolute')
    c1=rjadd(farg);
    D=D-min(min(D)) + c1;
end

out=surf(X,Y,Z,D);

%set(out,'EdgeColor','flat')    %Doesn't work with texturemapping, so I took it away

if initialhold==0;hold off;end
if size(colormap,1)>1
    caxis([1 size(colormap,1)])
end








